# -*- coding: utf-8 -*-
"""
Created on Thu Jun 13 13:36:59 2019

@author: WillemsenPWJM
"""

import csv
import matplotlib.pyplot as plt
import matplotlib.dates as md
import numpy as np
import pandas as pd
from pathlib import Path
import time
import datetime as dt

#import ased_script_library as ased

##############################################################################
############### INITIALISE ###################################################
##############################################################################

# manually add parameter values 
# showcase for paper uses '313-000A'
experiment_name = '333-013A'  
data_folder = r'E:\ASED\Marconi\RawData\20210701\333'
SALINITY = 25       # assumed salinity 
initial_HAB = 0.35  # Initial height above bottom of sensor head
L_bnd = 750         # lower boundary of raw signal in # of timesteps for search window
U_bnd = 3000        # upper boundary of raw signal in # of timesteps for search window
KF_ratio = 0.99     # ratio for Kalman filter for smoothing data
noval = -99999      # integer for not a value
dat_measpburst = 8  # number of measurements per burst
time_offset = 7200  # offset of measurement time in seconds (ASED is two hours behind)

# construct file and folder names
filename = data_folder + '/' + experiment_name + ".SE2.sedsonar"
output_dir_name = data_folder + "/output/" + experiment_name + ".txt"

# inital calculations
delta_seal = 0.004
speed_of_sound = 1500.235
timestep = 250e-09
negl_tstep = 533 + 80 # first 0.1m = 533 timesteps and 80 timesteps are not recorded
HAB = initial_HAB + delta_seal
HAB_time = (HAB / (timestep * speed_of_sound)) * 2 - negl_tstep
timestamp_Tref = dt.datetime(2001,1,1,0,0,0)
timestamp_Tref_ts = time.mktime(timestamp_Tref.timetuple()) 

# read data 
dat = pd.read_csv(filename, sep='\t', skiprows=1, header=None)
dat_Tburst_ts = dat.values[::8,0] + timestamp_Tref_ts # get time of measurement burst (=since 01-01-2001) + diff universal start time  (=since 01-01-1970)
dat_temperature = dat.values[:,2]           # temperature    
dat_pressurebar = dat.values[:,3] / 10000   # pressure in bar
dat_bedlevraw = dat.values[:,20:]           # raw data for bedlevel      
dat_stopmv = dat.values[:,15]               # measurement voltage      

# calculate waterlevel from pressure sensor
# atm_pressurebar = 1.01325                   # standard average atmospheric pressure  
# dat_waterlevel = (dat_pressurebar - atm_pressurebar) / 1000 / 9.81

##############################################################################
############### Process unique measurements ##################################
##############################################################################
dat_bedlevabs = np.absolute(dat_bedlevraw)  # absolute value of raw data
dat_maxval = np.amax(dat_bedlevabs[:,L_bnd:U_bnd], axis=1) # find maximum value in search window to discover peak
dat_maxval2 = np.amax(dat_bedlevabs[:,0:L_bnd], axis=1)
val_filtdry = 50
val_filt_ininoise = 40

#dat_maxval_bool = dat_maxval<val_filtdry # without initial noise filter
dat_maxval_bool = np.logical_and(dat_maxval<val_filtdry, dat_maxval2>val_filt_ininoise) # with initial noise filter

#dat_maxval_ind = np.asarray(np.where(dat_maxval>val_filtdry)) # without initial noise filter
dat_maxval_ind = np.asarray(np.where(np.logical_and(dat_maxval>val_filtdry, dat_maxval2<val_filt_ininoise))) # with initial noise filter

dat_bedlevsort = dat_bedlevabs
dat_bedlevsort[dat_maxval_bool,:] = noval
dat_nummeas = np.size(dat_bedlevraw,0)

dat_bedlevsmo = np.zeros((np.size(dat_bedlevraw,0),np.size(dat_bedlevraw,1)),dtype='int') + noval
dat_maxvalsmo = np.zeros((np.size(dat_bedlevraw,0),1),dtype='int') + noval
dat_maxvalsmo_ind = np.zeros((np.size(dat_bedlevraw,0),1),dtype='int') + noval
dat_minvalsmo_ind = np.zeros((np.size(dat_bedlevraw,0),1),dtype='int') + noval
dat_bedlev_ind = np.zeros((np.size(dat_bedlevraw,0),1),dtype='float') + noval
dat_bedlev_time = np.zeros((np.size(dat_bedlevraw,0),1),dtype='float') + noval
dat_bedlev_dist = np.zeros((np.size(dat_bedlevraw,0),1),dtype='float') + noval

def kalman_filter(old, new, ratio):
    return old * ratio + new * (1-ratio)

def locate_min(a):
    smallest = min(a)
    return smallest, [index for index, element in enumerate(a) 
                      if smallest == element]

for x in range(dat_nummeas):
    if x in dat_maxval_ind:
        temp_signal = dat_bedlevsort[x,:]
        initial = temp_signal[0]
        smoothed = [temp_signal[0]]
        for m in temp_signal:
            initial = kalman_filter(initial, m, 0.99)
            smoothed.append(initial)
        dat_bedlevsmo[x,:] = smoothed[1:] # matrix with smoothed data using kalman filter
        
        dat_maxvalsmo[x] =  np.amax(dat_bedlevsmo[x,L_bnd:U_bnd]) # find max value in search window         
        dat_maxvalsmo_ind[x] = np.argmax(dat_bedlevsmo[x,L_bnd:U_bnd]) + L_bnd # find index of max value in search window
        dat_minvalsmo_ind[x] = dat_maxvalsmo_ind[x] - \
        np.argmin(np.flipud(dat_bedlevsmo[x,L_bnd:np.asscalar(dat_maxvalsmo_ind[x])])) # find min value between lower boundary and peak (i.e. max val)
        dat_bedlev_ind[x] = dat_minvalsmo_ind[x] + negl_tstep 
        dat_bedlev_time[x] = dat_bedlev_ind[x] * timestep             
        dat_bedlev_dist[x] = dat_bedlev_time[x] * speed_of_sound / 2

##############################################################################
############### Filter and average per burst #################################
##############################################################################
dat_Tburst_dt=[dt.datetime.fromtimestamp(ts) for ts in dat_Tburst_ts]
dat_bedlev_dist_burstmatrix = np.reshape(dat_bedlev_dist,(dat_nummeas/dat_measpburst,dat_measpburst))
dat_stopmv_burstmatrix = np.reshape(dat_stopmv,(dat_nummeas/dat_measpburst,dat_measpburst))
dat_stopmv_burstmatrix.astype(float)
dat_burstsumpre = np.sum(dat_bedlev_dist_burstmatrix,axis=1)
# Filter incomplete bursts 
for x in range(dat_nummeas/dat_measpburst):
    if dat_burstsumpre[x] < 0:
        dat_bedlev_dist_burstmatrix[x,:] = np.NaN    
dat_stopmvpburst = np.median(dat_stopmv_burstmatrix,axis=1)
for x in range(dat_nummeas/dat_measpburst):
    if dat_burstsumpre[x] < 0:
        dat_stopmvpburst[x] = np.NaN

# Quality parameters 
dat_burststddev = np.std(dat_bedlev_dist_burstmatrix,axis=1)
dat_burstmin = np.min(dat_bedlev_dist_burstmatrix,axis=1)
dat_burstmax = np.max(dat_bedlev_dist_burstmatrix,axis=1)

# Filter too large standard deviation in burst 
for x in range(dat_nummeas/dat_measpburst):
    if dat_burststddev[x] > 0.002:
        dat_bedlev_dist_burstmatrix[x,:] = np.NaN  
        dat_burststddev[x] = np.NaN

# Median of burst data
dat_bedlev_distpburst = np.median(dat_bedlev_dist_burstmatrix,axis=1)
    
dat_bedlev_stddevpburst = np.zeros((dat_nummeas/dat_measpburst,2),dtype='float')
dat_bedlev_stddevpburst[:,0] = dat_bedlev_distpburst-dat_burststddev
dat_bedlev_stddevpburst[:,1] = dat_bedlev_distpburst+dat_burststddev
dat_bedlev_stddevpburst = np.transpose(dat_bedlev_stddevpburst)

dat_bedlev_minmaxpburst = np.zeros((dat_nummeas/dat_measpburst,2),dtype='float')
dat_bedlev_minmaxpburst[:,0] = dat_burstmin
dat_bedlev_minmaxpburst[:,1] = dat_burstmax
dat_bedlev_minmaxpburst = np.transpose(dat_bedlev_minmaxpburst)


dat_T_stddevpburst = np.zeros((dat_nummeas/dat_measpburst,2),dtype='float')
dat_T_stddevpburst[:,0] = dat_Tburst_ts
dat_T_stddevpburst[:,1] = dat_Tburst_ts
dat_T_stddevpburst = np.transpose(dat_T_stddevpburst)


##############################################################################
############### Create output ################################################
##############################################################################

# Figure of bed level change over time
fig1 = plt.figure()
ax = plt.gca() 
plt.plot(dat_Tburst_dt,dat_bedlev_distpburst,'.-')

xfmt = md.DateFormatter('%m-%d')
ax.xaxis.set_major_formatter(xfmt)
plt.grid(True)
plt.xlabel('time in 2019 (mm-dd)')
plt.ylabel('bed level referred to initial HAB (m)')
plt.ylim(0.20,0.60)
plt.rcParams.update({'font.size': 12})

# Figure with subplot bed level change and subplot stopmv
fig2 = plt.figure()
ax1 = fig2.add_subplot(211)
ax1 = plt.gca() 
plt.plot(dat_Tburst_dt,dat_bedlev_distpburst,'.-')
xfmt = md.DateFormatter('%m-%d')
ax1.xaxis.set_major_formatter(xfmt)
plt.grid(True)
plt.ylabel('bed level referred to initial HAB (m)')
plt.ylim(0.35,0.38)
ax1.tick_params(labelbottom=False)    
plt.rcParams.update({'font.size': 12})

ax2 = fig2.add_subplot(212,sharex=ax1)
ax2 = plt.gca() 
plt.plot(dat_Tburst_dt,dat_stopmvpburst,'.-')
xfmt = md.DateFormatter('%m-%d')
ax1.xaxis.set_major_formatter(xfmt)
plt.grid(True)
plt.ylabel('stopmv (mv)')
plt.xlabel('time in 2019 (mm-dd)')
plt.ylim(1600,12000)
plt.rcParams.update({'font.size': 12})

# Figure of bed level change vs stopmv
fig3 = plt.figure()
ax = plt.gca() 
plt.plot(dat_stopmvpburst,dat_bedlev_distpburst,'.')
plt.grid(True)
plt.xlabel('stopmv (mv)')
plt.ylabel('bed level referred to initial HAB (m)')
plt.rcParams.update({'font.size': 12})

# Figure of bed level change with standard deviation in background
fig4 = plt.figure()
ax = plt.gca() 
#plt.plot(dat_T_stddevpburst,dat_bedlev_minmaxpburst,'r')
plt.plot(dat_T_stddevpburst,dat_bedlev_stddevpburst,'k')
plt.plot(dat_Tburst_ts,dat_bedlev_distpburst,'.')
plt.grid(True)
plt.xlabel('time')
plt.ylabel('bed level referred to initial HAB (m)')
plt.ylim(0.4,0.5)
plt.rcParams.update({'font.size': 12})

# Figure of a random raw signal where a bed is found
examp = 1000
fig5 = plt.figure()
ax = plt.gca()
plt.plot(dat_bedlevraw[examp,:],label='raw')
plt.plot(dat_bedlevabs[examp,:],label='absolute value')
plt.plot(dat_bedlevsmo[examp,:],label='Kalman smoothed')
#plt.plot(dat_maxvalsmo_ind[examp],dat_maxvalsmo[examp])
plt.plot([dat_maxvalsmo_ind[examp],dat_maxvalsmo_ind[examp]],[-100,100],'--',color=[0.3,0.3,0.3],label='peak')
plt.plot([dat_minvalsmo_ind[examp],dat_minvalsmo_ind[examp]],[-100,100],'--r',label='local minimum')
plt.plot([5,1000],[-82.5,-82.5],'k')
plt.plot([5,5],[-80,-85],'k')
plt.plot([1000,1000],[-80,-85],'k')
plt.text(100,-95,'initial noise')
plt.plot([1250,2500],[-82.5,-82.5],'k')
plt.plot([1250,1250],[-80,-85],'k')
plt.plot([2500,2500],[-80,-85],'k')
plt.text(1350,-95,'bed reflection')
plt.grid(True)
plt.xlim([0,4000])
plt.ylim([-100,100])
plt.xlabel('time (250ns)')
plt.ylabel('intensity (1.15$\mu$V)')
plt.rcParams.update({'font.size': 12})
plt.legend(loc='upper right',fontsize=10)



fig5.set_size_inches(6,4)
plt.tight_layout()
plt.savefig("C:\Users\WillemsenPWJM\OneDrive\Documenten\PostDoc\Manuscript_Development\ASED_Marconi\Figures\Figure_ASEDsignalAppendix.png",
            dpi=150)

# outputmatrix
output = np.zeros((dat_nummeas/dat_measpburst,2),dtype='float')
output[:,0] = dat_Tburst_ts 
output[:,1] = dat_bedlev_distpburst 

np.savetxt(output_dir_name, output, delimiter=',')
