#!/usr/bin/env python

import sys, os, time, struct, getopt

SEC2001 = 978307200

def convert_temperature(fields):
	fields[0] = ( 60. - -10. ) * fields[0] / 2047. + -10.

def convert_sync(fields):
	sec = fields.pop(0)
	msec = fields.pop(0)
	fields.insert(0, SEC2001 + sec + msec / 1000. )
 
def convert_pressure(fields):
	fields[0] /= 10000.
 
def process_params(buf,o):
	o.write ( "parameters:\n" )
	o.write ( buf )

recordtypes = {
	'params': {
		"length": 1024,
		"suffix": "params",
		"cb": process_params
	},
	'SY': {
		'suffix': 'sync',
		'struct': struct.Struct(">IHHII"),
		'fields': [ 'sec', 'msec', 'flags', 'pressure', 'sedsonar' ]
	},
	'P': {
		'suffix': 'pressure',
		'struct': struct.Struct(">H"),
		'fields': [ 'pressure' ],
		'na': [ '0' ]
	},
	'SEDSONAR': {
		'suffix': 'sedsonar',
		'struct': struct.Struct("<iHhIIIhhhhhhHHHHHHHH4000b"),
		'fields': [ 'sec', 'msec', 'temp', 'pressure', 'run_ns', 'rf_ns', 'ax', 'ay', 'az', 'mx', 'my', 'mz', 'dac0', 'battery', 'startmv', 'stopmv', 'deadtime', 'flags', 'txadj', 'mark', 'buf' ],
	}
}

def process_file(fname, dumptostdout):
	print "processing", fname
	input_file = open(fname, 'rb')

	rt = recordtypes['params']
	if dumptostdout:
		rt['fd'] = sys.stdout
	else:
		rt['fd'] = open ( fname + "." + rt['suffix'], 'wb' )
		
	buf = ""
	while True:
		c = input_file.read(1)
		if ord(c) == 0:
			break
		else:
			buf = buf + c

	recordtypes['params']['cb'] (buf, recordtypes['params']['fd'] )

	while True:
		buf = ""
		for i in range(1, 10):
			c = input_file.read(1)
			if not c:
				break
			buf = buf + c

			if buf in recordtypes:
				break

		if not buf:
			break

		if buf not in recordtypes:
			raise Exception("invalid record type ", buf, input_file.tell())

		rt = recordtypes[buf]

		if 'struct' in rt:
			l = rt['struct'].size
		else:
			l = rt['length']

		buf = input_file.read ( l )

		if 'cb' in rt:
			fields = rt['cb'] ( buf )
		elif 'struct' in rt:
			fields = list(rt['struct'].unpack(buf))
			if 'convert' in rt:
				rt['convert'] ( fields )
			if 'na' in rt:
				for i in xrange(len(rt['na'])):
					if str(fields[i]) == rt['na'][i]:
						fields[i] = 'NA'
					
		else:
			raise Exception("don't known what to do!!!")

		if 'fd' not in rt:
			if dumptostdout:
				rt['fd'] = sys.stdout
			else:
				rt['fd'] = open ( fname + "." + rt['suffix'], 'wb' )
				if 'fields' in rt:
					rt['fd'].write ( "\t".join ( rt['fields'] ) + "\n" )

		if dumptostdout:
			fields.insert(0, rt['suffix'])

		rt['fd'].write ( "\t".join(map(str, fields)) + "\n" )

	print "done"

	if not dumptostdout:
		for t,v in recordtypes.iteritems():
			if 'fd' in v:
				v['fd'].close()
				del v['fd']


def main():
	dumptostdout = False

	opts, files = getopt.gnu_getopt(sys.argv[1:], "s")

	for o,v in opts:
		if o == "-s":
			dumptostdout = True

	for fname in files:
		process_file(fname, dumptostdout)

try:
	main()
except KeyboardInterrupt:
	pass

